@@ -21,9 +21,9 @@
 
 #include <qtest.h>
 
-#include "kimap2test/fakeserver.h"
-#include "kimap2/session.h"
-#include "kimap2/capabilitiesjob.h"
+#include "kimaptest/fakeserver.h"
+#include "kimap/session.h"
+#include "kimap/capabilitiesjob.h"
 
 #include <QtTest>
 
@@ -44,7 +44,7 @@ private Q_SLOTS:
                  << "S: A000001 OK CAPABILITY completed";
 
         QStringList capabilities;
-        capabilities << QStringLiteral("IMAP4REV1") << QStringLiteral("STARTTLS") <<  QStringLiteral("AUTH=GSSAPI");
+        capabilities << QLatin1String("IMAP4REV1") << QLatin1String("STARTTLS") <<  QLatin1String("AUTH=GSSAPI");
         QTest::newRow("good") << scenario << capabilities;
 
         scenario.clear();
@@ -62,7 +62,7 @@ private Q_SLOTS:
                  << "S: * some response"
                  << "S: A000001 OK CAPABILITY completed";
 
-        capabilities << QStringLiteral("IMAP4REV1") << QStringLiteral("STARTTLS") <<  QStringLiteral("AUTH=PLAIN");
+        capabilities << QLatin1String("IMAP4REV1") << QLatin1String("STARTTLS") <<  QLatin1String("AUTH=PLAIN");
         QTest::newRow("extra-untagged") << scenario << capabilities;;
     }
 
@@ -74,9 +74,9 @@ private Q_SLOTS:
         FakeServer fakeServer;
         fakeServer.setScenario(scenario);
         fakeServer.startAndWait();
-        KIMAP2::Session session(QStringLiteral("127.0.0.1"), 5989);
+        KIMAP::Session session(QLatin1String("127.0.0.1"), 5989);
 
-        KIMAP2::CapabilitiesJob *job = new KIMAP2::CapabilitiesJob(&session);
+        KIMAP::CapabilitiesJob *job = new KIMAP::CapabilitiesJob(&session);
         bool result = job->exec();
         QEXPECT_FAIL("bad" , "Expected failure on BAD response", Continue);
         QVERIFY(result);
