@@ -21,9 +21,9 @@
 
 #include <qtest.h>
 
-#include "kimap2test/fakeserver.h"
-#include "kimap2/session.h"
-#include "kimap2/storejob.h"
+#include "kimaptest/fakeserver.h"
+#include "kimap/session.h"
+#include "kimap/storejob.h"
 
 #include <QtTest>
 
@@ -60,16 +60,6 @@ private Q_SLOTS:
         QTest::newRow("uid based") << true << qint64(3) << qint64(1096)
                                    << (QList<QByteArray>() << "\\Seen" << "\\Foo")
                                    << scenario;
-
-        scenario.clear();
-        scenario << FakeServer::preauth()
-                 << "C: A000001 UID STORE 1096 FLAGS ()"
-                 << "S: * 3 FETCH (FLAGS () UID 1096)"
-                 << "S: A000001 OK STORE completed";
-
-        QTest::newRow("set emtpy flag set") << true << qint64(3) << qint64(1096)
-                                   << QList<QByteArray>()
-                                   << scenario;
     }
 
     void testStore()
@@ -84,13 +74,13 @@ private Q_SLOTS:
         fakeServer.setScenario(scenario);
         fakeServer.startAndWait();
 
-        KIMAP2::Session session(QStringLiteral("127.0.0.1"), 5989);
+        KIMAP::Session session(QLatin1String("127.0.0.1"), 5989);
 
-        KIMAP2::StoreJob *job = new KIMAP2::StoreJob(&session);
+        KIMAP::StoreJob *job = new KIMAP::StoreJob(&session);
         job->setUidBased(uidBased);
-        job->setSequenceSet(KIMAP2::ImapSet(uidBased ? uid : id));
+        job->setSequenceSet(KIMAP::ImapSet(uidBased ? uid : id));
         job->setFlags(flags);
-        job->setMode(KIMAP2::StoreJob::SetFlags);
+        job->setMode(KIMAP::StoreJob::SetFlags);
         bool result = job->exec();
         QVERIFY(result);
         if (uidBased) {
