@@ -19,12 +19,14 @@
 
 #include "appendjob.h"
 
+#include <KLocalizedString>
+
 #include "job_p.h"
 #include "message_p.h"
 #include "session_p.h"
 #include "rfccodecs.h"
 
-namespace KIMAP2
+namespace KIMAP
 {
 class AppendJobPrivate : public JobPrivate
 {
@@ -40,10 +42,10 @@ public:
 };
 }
 
-using namespace KIMAP2;
+using namespace KIMAP;
 
 AppendJob::AppendJob(Session *session)
-    : Job(*new AppendJobPrivate(session, "Append"))
+    : Job(*new AppendJobPrivate(session, i18n("Append")))
 {
 }
 
@@ -109,7 +111,7 @@ void AppendJob::doStart()
 {
     Q_D(AppendJob);
 
-    QByteArray parameters = '\"' + KIMAP2::encodeImapFolderName(d->mailBox.toUtf8()) + '\"';
+    QByteArray parameters = '\"' + KIMAP::encodeImapFolderName(d->mailBox.toUtf8()) + '\"';
 
     if (!d->flags.isEmpty()) {
         parameters += " (";
@@ -122,12 +124,12 @@ void AppendJob::doStart()
 
     if (!d->internalDate.isNull()) {
         const QDateTime utcDateTime = d->internalDate.toUTC();
-        parameters += " \"" + QLocale::c().toString(utcDateTime, QStringLiteral("dd-MMM-yyyy hh:mm:ss")).toLatin1() + " +0000" + '\"';
+        parameters += " \"" + utcDateTime.toString(QString::fromAscii("dd-MMM-yyyy hh:mm:ss")).toLatin1() + " +0000" + '\"';
     }
 
     parameters += " {" + QByteArray::number(d->content.size()) + '}';
 
-    d->sendCommand("APPEND", parameters);
+    d->tags << d->sessionInternal()->sendCommand("APPEND", parameters);
 }
 
 void AppendJob::handleResponse(const Message &response)
