@@ -20,12 +20,13 @@
 #include "idlejob.h"
 
 #include <QtCore/QTimer>
+#include <KLocalizedString>
 
 #include "job_p.h"
 #include "message_p.h"
 #include "session_p.h"
 
-namespace KIMAP2
+namespace KIMAP
 {
 class IdleJobPrivate : public JobPrivate
 {
@@ -70,10 +71,10 @@ public:
 };
 }
 
-using namespace KIMAP2;
+using namespace KIMAP;
 
 IdleJob::IdleJob(Session *session)
-    : Job(*new IdleJobPrivate(this, session, "Idle"))
+    : Job(*new IdleJobPrivate(this, session, i18nc("name of the idle job", "Idle")))
 {
     Q_D(IdleJob);
     connect(&d->emitStatsTimer, SIGNAL(timeout()),
@@ -87,7 +88,7 @@ IdleJob::~IdleJob()
 {
 }
 
-void KIMAP2::IdleJob::stop()
+void KIMAP::IdleJob::stop()
 {
     Q_D(IdleJob);
     d->sessionInternal()->setSocketTimeout(d->originalSocketTimeout);
@@ -99,7 +100,7 @@ void IdleJob::doStart()
     Q_D(IdleJob);
     d->originalSocketTimeout = d->sessionInternal()->socketTimeout();
     d->sessionInternal()->setSocketTimeout(-1);
-    d->sendCommand("IDLE", {});
+    d->tags << d->sessionInternal()->sendCommand("IDLE");
 }
 
 void IdleJob::handleResponse(const Message &response)
@@ -148,19 +149,19 @@ void IdleJob::handleResponse(const Message &response)
     }
 }
 
-QString KIMAP2::IdleJob::lastMailBox() const
+QString KIMAP::IdleJob::lastMailBox() const
 {
     Q_D(const IdleJob);
     return d->m_session->selectedMailBox();
 }
 
-int KIMAP2::IdleJob::lastMessageCount() const
+int KIMAP::IdleJob::lastMessageCount() const
 {
     Q_D(const IdleJob);
     return d->lastMessageCount;
 }
 
-int KIMAP2::IdleJob::lastRecentCount() const
+int KIMAP::IdleJob::lastRecentCount() const
 {
     Q_D(const IdleJob);
     return d->lastRecentCount;
