@@ -1,6 +1,5 @@
 /*
     Copyright (c) 2009 Kevin Ottens <ervin@kde.org>
-    Copyright (c) 2017 Christian Mollekopf <mollekopf@kolabsys.com>
 
     Copyright (c) 2010 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
     Author: Kevin Ottens <kevin@kdab.com>
@@ -23,101 +22,79 @@
 
 #include "session.h"
 #include "session_p.h"
+#include "sessionuiproxy.h"
 
-#include <QDebug>
+#include <QtCore/QDebug>
+#include <QtCore/QTimer>
 
 #include "kimap_debug.h"
 
 #include "job.h"
+#include "loginjob.h"
 #include "message_p.h"
 #include "sessionlogger_p.h"
+#include "sessionthread_p.h"
 #include "rfccodecs.h"
-#include "imapstreamparser.h"
 
-Q_DECLARE_METATYPE(QSsl::SslProtocol)
+Q_DECLARE_METATYPE(KTcpSocket::SslVersion)
 Q_DECLARE_METATYPE(QSslSocket::SslMode)
-static const int _kimap_sslVersionId = qRegisterMetaType<QSsl::SslProtocol>();
+static const int _kimap_sslVersionId = qRegisterMetaType<KTcpSocket::SslVersion>();
 
-using namespace KIMAP2;
+using namespace KIMAP;
 
 Session::Session(const QString &hostName, quint16 port, QObject *parent)
     : QObject(parent), d(new SessionPrivate(this))
 {
-    if (!qEnvironmentVariableIsEmpty("KIMAP2_LOGFILE")) {
-        d->logger.reset(new SessionLogger);
-        qCInfo(KIMAP2_LOG) << "Logging traffic to: " << QLatin1String(qgetenv("KIMAP2_LOGFILE"));
-    }
-    if (qEnvironmentVariableIsSet("KIMAP2_TRAFFIC")) {
-        d->dumpTraffic = true;
-        qCInfo(KIMAP2_LOG) << "Dumping traffic.";
-    }
-    if (qEnvironmentVariableIsSet("KIMAP2_TIMING")) {
-        d->trackTime = true;
-        qCInfo(KIMAP2_LOG) << "Tracking timings.";
+    if (!qgetenv("KIMAP_LOGFILE").isEmpty()) {
+        d->logger = new SessionLogger;
     }
 
     d->isSocketConnected = false;
     d->state = Disconnected;
     d->jobRunning = false;
-    d->hostName = hostName;
-    d->port = port;
 
-    connect(d->socket.data(), &QIODevice::readyRead, d, &SessionPrivate::readMessage);
-
-    connect(d->socket.data(), &QSslSocket::connected,
-            d, &SessionPrivate::socketConnected);
-    connect(d->socket.data(), static_cast<void (QSslSocket::*)(const QList<QSslError>&)>(&QSslSocket::sslErrors),
-            d, &SessionPrivate::handleSslErrors);
-    connect(d->socket.data(), static_cast<void (QSslSocket::*)(QAbstractSocket::SocketError)>(&QSslSocket::error),
-            d, &SessionPrivate::socketError);
-
-    connect(d->socket.data(), &QIODevice::bytesWritten,
-            d, &SessionPrivate::socketActivity);
-    connect(d->socket.data(), &QSslSocket::encryptedBytesWritten,
-            d, &SessionPrivate::socketActivity);
-    connect(d->socket.data(), &QIODevice::readyRead,
-            d, &SessionPrivate::socketActivity);
-    connect(d->socket.data(), &QAbstractSocket::stateChanged, [this](QAbstractSocket::SocketState state) {
-        qCDebug(KIMAP2_LOG) << "Socket state changed: " << state;
-        //The disconnected signal will not fire if we fail to lookup the host, but this will.
-        if (state == QAbstractSocket::UnconnectedState) {
-            d->socketDisconnected();
-        }
-        if (state == QAbstractSocket::HostLookupState) {
-            d->hostLookupInProgress = true;
-        } else {
-            d->hostLookupInProgress = false;
-        }
-    });
+    d->thread = new SessionThread(hostName, port);
+    connect(d->thread, &SessionThread::encryptionNegotiationResult, d, &SessionPrivate::onEncryptionNegotiationResult);
+    connect(d->thread, &SessionThread::sslError, d, &SessionPrivate::handleSslError);
+    connect(d->thread, &SessionThread::socketDisconnected, d, &SessionPrivate::socketDisconnected);
+    connect(d->thread, &SessionThread::responseReceived, d, &SessionPrivate::responseReceived);
+    connect(d->thread, &SessionThread::socketConnected, d, &SessionPrivate::socketConnected);
+    connect(d->thread, &SessionThread::socketActivity, d, &SessionPrivate::socketActivity);
+    connect(d->thread, &SessionThread::socketError, d, &SessionPrivate::socketError);
 
     d->socketTimer.setSingleShot(true);
-    connect(&d->socketTimer, &QTimer::timeout,
-            d, &SessionPrivate::onSocketTimeout);
-
-    d->socketProgressTimer.setSingleShot(false);
-    connect(&d->socketProgressTimer, &QTimer::timeout,
-            d, &SessionPrivate::onSocketProgressTimeout);
+    connect(&d->socketTimer, SIGNAL(timeout()),
+            d, SLOT(onSocketTimeout()));
 
     d->startSocketTimer();
-
-    QMetaObject::invokeMethod(d, "reconnect", Qt::QueuedConnection);
 }
 
 Session::~Session()
 {
     //Make sure all jobs know we're done
-    d->clearJobQueue();
-    delete d;
+    d->socketDisconnected();
+    delete d->thread;
+    d->thread = Q_NULLPTR;
+}
+
+void Session::setUiProxy(const SessionUiProxy::Ptr &proxy)
+{
+    d->uiProxy = proxy;
+}
+
+void Session::setUiProxy(SessionUiProxy *proxy)
+{
+    setUiProxy(SessionUiProxy::Ptr(proxy));
 }
 
 QString Session::hostName() const
 {
-    return d->hostName;
+    return d->thread->hostName();
 }
 
 quint16 Session::port() const
 {
-    return d->port;
+    return d->thread->port();
 }
 
 Session::State Session::state() const
@@ -125,11 +102,6 @@ Session::State Session::state() const
     return d->state;
 }
 
-bool Session::isConnected() const
-{
-    return (d->state == Authenticated || d->state == Selected);
-}
-
 QString Session::userName() const
 {
     return d->userName;
@@ -145,61 +117,35 @@ int Session::jobQueueSize() const
     return d->queue.size() + (d->jobRunning ? 1 : 0);
 }
 
-void Session::close()
+void KIMAP::Session::close()
 {
-    d->closeSocket();
+    d->thread->closeSocket();
 }
 
-void Session::ignoreErrors(const QList<QSslError> &errors)
+void SessionPrivate::handleSslError(const KSslErrorUiData &errorData)
 {
-    d->socket->ignoreSslErrors(errors);
+    const bool ignoreSslError = uiProxy && uiProxy->ignoreSslError(errorData);
+    //ignoreSslError is async, so the thread might already be gone when it returns
+    if (thread) {
+        thread->sslErrorHandlerResponse(ignoreSslError);
+    }
 }
 
-void Session::setTimeout(int timeout)
-{
-    d->setSocketTimeout(timeout * 1000);
-}
-
-int Session::timeout() const
-{
-    return d->socketTimeout() / 1000;
-}
-
-QString Session::selectedMailBox() const
-{
-    return QString::fromUtf8(d->currentMailBox);
-}
-
-
 SessionPrivate::SessionPrivate(Session *session)
     : QObject(session),
       q(session),
       state(Session::Disconnected),
-      hostLookupInProgress(false),
       logger(Q_NULLPTR),
       currentJob(Q_NULLPTR),
       tagCount(0),
-      socketTimerInterval(30000),   // By default timeouts on 30s
-      socketProgressInterval(3000),   // mention we're still alive every 3s
-      socket(new QSslSocket),
-      stream(new ImapStreamParser(socket.data())),
-      accumulatedWaitTime(0),
-      accumulatedProcessingTime(0),
-      trackTime(false),
-      dumpTraffic(false)
+      sslVersion(KTcpSocket::UnknownSslVersion),
+      socketTimerInterval(30000)   // By default timeouts on 30s
 {
-    stream->onResponseReceived([this](const Message &message) {
-        responseReceived(message);
-    });
 }
 
 SessionPrivate::~SessionPrivate()
 {
-}
-
-void SessionPrivate::handleSslErrors(const QList<QSslError> &errors)
-{
-    emit q->sslErrors(errors);
+    delete logger;
 }
 
 void SessionPrivate::addJob(Job *job)
@@ -207,9 +153,12 @@ void SessionPrivate::addJob(Job *job)
     queue.append(job);
     emit q->jobQueueSizeChanged(q->jobQueueSize());
 
-    QObject::connect(job, &KJob::result, this, &SessionPrivate::jobDone);
-    QObject::connect(job, &QObject::destroyed, this, &SessionPrivate::jobDestroyed);
-    startNext();
+    QObject::connect(job, SIGNAL(result(KJob*)), this, SLOT(jobDone(KJob*)));
+    QObject::connect(job, SIGNAL(destroyed(QObject*)), this, SLOT(jobDestroyed(QObject*)));
+
+    if (state != Session::Disconnected) {
+        startNext();
+    }
 }
 
 void SessionPrivate::startNext()
@@ -223,14 +172,10 @@ void SessionPrivate::doStartNext()
         return;
     }
 
-    if (trackTime) {
-        time.start();
-    }
     restartSocketTimer();
     jobRunning = true;
 
     currentJob = queue.dequeue();
-    qCDebug(KIMAP2_LOG) << "Starting job: " << currentJob->metaObject()->className();
     currentJob->doStart();
 }
 
@@ -238,7 +183,6 @@ void SessionPrivate::jobDone(KJob *job)
 {
     Q_UNUSED(job);
     Q_ASSERT(job == currentJob);
-    qCDebug(KIMAP2_LOG) << "Job done: " << job->metaObject()->className();
 
     stopSocketTimer();
 
@@ -250,7 +194,7 @@ void SessionPrivate::jobDone(KJob *job)
 
 void SessionPrivate::jobDestroyed(QObject *job)
 {
-    queue.removeAll(static_cast<KIMAP2::Job *>(job));
+    queue.removeAll(static_cast<KIMAP::Job *>(job));
     if (currentJob == job) {
         currentJob = Q_NULLPTR;
     }
@@ -258,10 +202,7 @@ void SessionPrivate::jobDestroyed(QObject *job)
 
 void SessionPrivate::responseReceived(const Message &response)
 {
-    if (dumpTraffic) {
-        qCInfo(KIMAP2_LOG) << "S: " << QString::fromLatin1(response.toString());
-    }
-    if (logger && q->isConnected()) {
+    if (logger && (state == Session::Authenticated || state == Session::Selected)) {
         logger->dataReceived(response.toString());
     }
 
@@ -284,28 +225,35 @@ void SessionPrivate::responseReceived(const Message &response)
             simplified.content.removeFirst(); // Strip the tag
             simplified.content.removeFirst(); // Strip the code
         }
-        qCDebug(KIMAP2_LOG) << "Received BYE: " << simplified.toString();
+        qCDebug(KIMAP_LOG) << "Received BYE: " << simplified.toString();
         return;
     }
 
     switch (state) {
     case Session::Disconnected:
-        stopSocketTimer();
+        if (socketTimer.isActive()) {
+            stopSocketTimer();
+        }
         if (code == "OK") {
-            Message simplified = response;
-            simplified.content.removeFirst(); // Strip the tag
-            simplified.content.removeFirst(); // Strip the code
-            greeting = simplified.toString().trimmed(); // Save the server greeting
             setState(Session::NotAuthenticated);
-        } else if (code == "PREAUTH") {
+
             Message simplified = response;
             simplified.content.removeFirst(); // Strip the tag
             simplified.content.removeFirst(); // Strip the code
             greeting = simplified.toString().trimmed(); // Save the server greeting
+
+            startNext();
+        } else if (code == "PREAUTH") {
             setState(Session::Authenticated);
+
+            Message simplified = response;
+            simplified.content.removeFirst(); // Strip the tag
+            simplified.content.removeFirst(); // Strip the code
+            greeting = simplified.toString().trimmed(); // Save the server greeting
+
+            startNext();
         } else {
-            //We have been rejected
-            closeSocket();
+            thread->closeSocket();
         }
         return;
     case Session::NotAuthenticated:
@@ -341,11 +289,11 @@ void SessionPrivate::responseReceived(const Message &response)
     }
 
     // If a job is running forward it the response
-    if (currentJob) {
+    if (currentJob != Q_NULLPTR) {
         restartSocketTimer();
         currentJob->handleResponse(response);
     } else {
-        qCWarning(KIMAP2_LOG) << "A message was received from the server with no job to handle it:"
+        qCWarning(KIMAP_LOG) << "A message was received from the server with no job to handle it:"
                              << response.toString()
                              << '(' + response.toString().toHex() + ')';
     }
@@ -378,7 +326,7 @@ QByteArray SessionPrivate::sendCommand(const QByteArray &command, const QByteArr
         upcomingMailBox = args;
         upcomingMailBox.remove(0, 1);
         upcomingMailBox = upcomingMailBox.left(upcomingMailBox.indexOf('\"'));
-        upcomingMailBox = KIMAP2::decodeImapFolderName(upcomingMailBox);
+        upcomingMailBox = KIMAP::decodeImapFolderName(upcomingMailBox);
     } else if (command == "CLOSE") {
         closeTag = tag;
     }
@@ -389,83 +337,94 @@ void SessionPrivate::sendData(const QByteArray &data)
 {
     restartSocketTimer();
 
-    if (dumpTraffic) {
-        qCInfo(KIMAP2_LOG) << "C: " << data;
-    }
-    if (logger && q->isConnected()) {
+    if (logger && (state == Session::Authenticated || state == Session::Selected)) {
         logger->dataSent(data);
     }
 
-    dataQueue.enqueue(data + "\r\n");
-    QMetaObject::invokeMethod(this, "writeDataQueue");
+    thread->sendData(data + "\r\n");
 }
 
 void SessionPrivate::socketConnected()
 {
-    qCInfo(KIMAP2_LOG) << "Socket connected.";
+    stopSocketTimer();
     isSocketConnected = true;
-    startNext();
+
+    bool willUseSsl = false;
+    if (!queue.isEmpty()) {
+        KIMAP::LoginJob *login = qobject_cast<KIMAP::LoginJob *>(queue.first());
+        if (login) {
+            willUseSsl = (login->encryptionMode() == KIMAP::LoginJob::SslV2) ||
+                         (login->encryptionMode() == KIMAP::LoginJob::SslV3) ||
+                         (login->encryptionMode() == KIMAP::LoginJob::SslV3_1) ||
+                         (login->encryptionMode() == KIMAP::LoginJob::AnySslVersion);
+
+            userName = login->userName();
+        }
+    }
+
+    if (state == Session::Disconnected && willUseSsl) {
+        startNext();
+    } else {
+        startSocketTimer();
+    }
 }
 
 void SessionPrivate::socketDisconnected()
 {
-    qCInfo(KIMAP2_LOG) << "Socket disconnected." << isSocketConnected;
-    stopSocketTimer();
+    if (socketTimer.isActive()) {
+        stopSocketTimer();
+    }
 
-    if (logger && q->isConnected()) {
+    if (logger && (state == Session::Authenticated || state == Session::Selected)) {
         logger->disconnectionOccured();
     }
 
-    isSocketConnected = false;
-
     if (state != Session::Disconnected) {
         setState(Session::Disconnected);
+        emit q->connectionLost();
     } else {
-        //If we timeout during host lookup we don't receive an explicit host lookup error
-        if (hostLookupInProgress) {
-            socketError(QAbstractSocket::HostNotFoundError);
-            hostLookupInProgress = false;
-        }
         emit q->connectionFailed();
     }
 
+    isSocketConnected = false;
+
     clearJobQueue();
 }
 
 void SessionPrivate::socketActivity()
 {
-    //This slot can be called after the job has already finished, in that case we don't want to restart the timer
-    if (currentJob) {
-        restartSocketTimer();
-    }
+    restartSocketTimer();
 }
 
-void SessionPrivate::socketError(QAbstractSocket::SocketError error)
+void SessionPrivate::socketError(KTcpSocket::Error error)
 {
-    qCDebug(KIMAP2_LOG) << "Socket error: " << error;
-    stopSocketTimer();
+    if (socketTimer.isActive()) {
+        stopSocketTimer();
+    }
 
     if (currentJob) {
-        qCWarning(KIMAP2_LOG) << "Socket error:" << error;
         currentJob->setSocketError(error);
     } else if (!queue.isEmpty()) {
-        qCWarning(KIMAP2_LOG) << "Socket error:" << error;
         currentJob = queue.takeFirst();
         currentJob->setSocketError(error);
     }
 
     if (isSocketConnected) {
-        closeSocket();
+        thread->closeSocket();
+    } else {
+        emit q->connectionFailed();
+        emit q->connectionLost();    // KDE5: Remove this. We shouldn't emit connectionLost() if we weren't connected in the first place
+        clearJobQueue();
     }
 }
 
 void SessionPrivate::clearJobQueue()
 {
-    if (!currentJob && !queue.isEmpty()) {
-        currentJob = queue.takeFirst();
-    }
     if (currentJob) {
         currentJob->connectionLost();
+    } else if (!queue.isEmpty()) {
+        currentJob = queue.takeFirst();
+        currentJob->connectionLost();
     }
 
     QQueue<Job *> queueCopy = queue; // copy because jobDestroyed calls removeAll
@@ -474,23 +433,29 @@ void SessionPrivate::clearJobQueue()
     emit q->jobQueueSizeChanged(0);
 }
 
-void SessionPrivate::startSsl(QSsl::SslProtocol protocol)
+void SessionPrivate::startSsl(const KTcpSocket::SslVersion &version)
 {
-    socket->setProtocol(protocol);
-    connect(socket.data(), &QSslSocket::encrypted, this, &SessionPrivate::sslConnected);
-    if (socket->state() == QAbstractSocket::ConnectedState) {
-        qCDebug(KIMAP2_LOG) << "Starting client encryption";
-        Q_ASSERT(socket->mode() == QSslSocket::UnencryptedMode);
-        socket->startClientEncryption();
-    } else {
-        qCWarning(KIMAP2_LOG) << "The socket is not yet connected";
-    }
+    thread->startSsl(version);
 }
 
-void SessionPrivate::sslConnected()
+QString Session::selectedMailBox() const
 {
-    qCDebug(KIMAP2_LOG) << "ssl is connected";
-    emit encryptionNegotiationResult(true);
+    return QString::fromUtf8(d->currentMailBox);
+}
+
+void SessionPrivate::onEncryptionNegotiationResult(bool isEncrypted, KTcpSocket::SslVersion version)
+{
+    if (isEncrypted) {
+        sslVersion = version;
+    } else {
+        sslVersion = KTcpSocket::UnknownSslVersion;
+    }
+    emit encryptionNegotiationResult(isEncrypted);
+}
+
+KTcpSocket::SslVersion SessionPrivate::negotiatedEncryption() const
+{
+    return sslVersion;
 }
 
 void SessionPrivate::setSocketTimeout(int ms)
@@ -521,104 +486,39 @@ void SessionPrivate::startSocketTimer()
     Q_ASSERT(!socketTimer.isActive());
 
     socketTimer.start(socketTimerInterval);
-    socketProgressTimer.start(socketProgressInterval);
 }
 
 void SessionPrivate::stopSocketTimer()
 {
+    if (socketTimerInterval < 0) {
+        return;
+    }
+
     socketTimer.stop();
-    socketProgressTimer.stop();
 }
 
 void SessionPrivate::restartSocketTimer()
 {
-    stopSocketTimer();
+    if (socketTimer.isActive()) {
+        stopSocketTimer();
+    }
     startSocketTimer();
 }
 
 void SessionPrivate::onSocketTimeout()
 {
-    qCWarning(KIMAP2_LOG) << "Aborting on socket timeout. " << socketTimerInterval;
-    if (!currentJob && !queue.isEmpty()) {
-        currentJob = queue.takeFirst();
-    }
-    if (currentJob) {
-        qCWarning(KIMAP2_LOG) << "Current job: " << currentJob->metaObject()->className();
-        currentJob->setErrorMessage("Aborting on socket timeout. Interval " + QString::number(socketTimerInterval) + " ms");
-    }
-    socket->abort();
-    socketProgressTimer.stop();
+    qCDebug(KIMAP_LOG) << "Socket timeout!";
+    thread->closeSocket();
 }
 
-QString SessionPrivate::getStateName() const 
+void Session::setTimeout(int timeout)
 {
-    if (hostLookupInProgress) {
-        return "Host lookup";
-    }
-    switch (state) {
-        case Session::Disconnected:
-            return "Disconnected";
-        case Session::NotAuthenticated:
-            return "NotAuthenticated";
-        case Session::Authenticated:
-            return "Authenticated";
-        case Session::Selected:
-        default:
-            break;
-    }
-    return "Unknown State";
+    d->setSocketTimeout(timeout * 1000);
 }
 
-void SessionPrivate::onSocketProgressTimeout()
+int Session::timeout() const
 {
-    if (currentJob) {
-        qCDebug(KIMAP2_LOG) << "Processing job: " << currentJob->metaObject()->className() << "Current state: " << getStateName() << (socket ? socket->state() : QAbstractSocket::UnconnectedState);
-    } else {
-        qCDebug(KIMAP2_LOG) << "Next job: " << (queue.isEmpty() ? "No job" : queue.head()->metaObject()->className()) << "Current state: " << getStateName() << (socket ? socket->state() : QAbstractSocket::UnconnectedState);
-    }
-}
-
-void SessionPrivate::writeDataQueue()
-{
-    while (!dataQueue.isEmpty()) {
-        socket->write(dataQueue.dequeue());
-    }
-}
-
-void SessionPrivate::readMessage()
-{
-    if (trackTime) {
-        accumulatedWaitTime += time.elapsed();
-        time.start();
-    }
-    stream->parseStream();
-    if (stream->error()) {
-        qCWarning(KIMAP2_LOG) << "Error while parsing, closing connection.";
-        qCDebug(KIMAP2_LOG) << "Current buffer: " << stream->currentBuffer();
-        socket->close();
-    }
-    if (trackTime) {
-        accumulatedProcessingTime += time.elapsed();
-        time.start();
-        qCDebug(KIMAP2_LOG) << "Wait vs process vs total: " << accumulatedWaitTime << accumulatedProcessingTime << accumulatedWaitTime + accumulatedProcessingTime;
-    }
-}
-
-void SessionPrivate::closeSocket()
-{
-    qCDebug(KIMAP2_LOG) << "Closing socket.";
-    socket->close();
-}
-
-void SessionPrivate::reconnect()
-{
-    if (socket->state() == QSslSocket::ConnectedState &&
-        socket->state() == QSslSocket::ConnectingState) {
-        qCDebug(KIMAP2_LOG) << "Reconnecting to: " << hostName << port;
-    } else {
-        qCDebug(KIMAP2_LOG) << "Connecting to: " << hostName << port;
-    }
-    socket->connectToHost(hostName, port);
+    return d->socketTimeout() / 1000;
 }
 
 #include "moc_session.cpp"
